from typing import List, Union

from minds.datasources import Datasource


class Mind:
    def __init__(
        self, client, name,
        model_name=None,
        provider=None,
        parameters=None,
        datasources=None,
        created_at=None,
        updated_at=None,
        **kwargs
    ):
        self.api = client.api
        self.client = client
        self.project = 'mindsdb'

        self.name = name
        self.model_name = model_name
        self.provider = provider
        self.parameters = parameters
        self.created_at = created_at
        self.updated_at = updated_at

        self.datasources = []
        for name in datasources:
            try:
                ds = self.client.datasources.get(name)
            except RuntimeError:
                # TODO skipped, it could be not sql skill
                continue
            self.datasources.append(ds)

    def update(
        self,
        model_name=None,
        provider=None,
        parameters=None,
        datasources=None
    ):
        if datasources:
            datasources = [ds.model_dump() for ds in datasources]

        self.api.patch(
            f'/projects/{self.project}/minds',
            data={
                'name': self.name,
                'model_name': model_name,
                'provider': provider,
                'parameters': parameters,
                'datasources': datasources,
            }
        )

    def add_datasource(self, datasource: Datasource):
        self.api.post(
            f'/projects/{self.project}/minds/{self.name}/datasources',
            data=datasource.model_dump()
        )
        self.datasources.append(datasource)

    def del_datasource(self, datasource: Union[Datasource, str]):
        raise NotImplemented


class Minds:
    def __init__(self, client):
        self.api = client.api
        self.client = client

        self.project = 'mindsdb'

    def list(self) -> List[Mind]:
        data = self.api.get(f'/projects/{self.project}/minds').json()
        minds_list = []
        for item in data:
            minds_list.append(Mind(self.client, **item))
        return minds_list

    def get(self, name: str) -> Mind:
        item = self.api.get(f'/projects/{self.project}/minds/{name}').json()
        return Mind(self.client, **item)

    def create(
        self, name,
        model_name=None,
        provider=None,
        parameters=None,
        datasources=None,
        replace=False,
    ) -> Mind:

        if replace:
            try:
                self.drop(name)
            except Exception:
                ...
        if datasources:
            datasources = [ds.model_dump() for ds in datasources]
        else:
            datasources = []

        self.api.post(
            f'/projects/{self.project}/minds',
            data={
                'name': name,
                'model_name': model_name,
                'provider': provider,
                'parameters': parameters,
                'datasources': datasources,
            }
        )
        mind = self.get(name)

        return mind

    def drop(self, name: str):
        mind = self.get(name)
        for ds in mind.datasources:
            mind.del_datasource(ds)

        self.api.delete(f'/projects/{self.project}/minds/{name}')



from typing import List, Optional, Union

from pydantic import BaseModel, Field


class DatabaseConfig(BaseModel):

    name: str
    engine: str
    description: str
    connection_data: Union[dict, None] = {}
    tables: Union[List[str], None] = []


class Datasource(DatabaseConfig):
    ...


class Datasources:
    def __init__(self, client):
        self.api = client.api

    def create(self, ds_config: DatabaseConfig):
        name = ds_config.name
        self.api.post('/datasources', data=ds_config.model_dump())
        return self.get(name)

    def list(self) -> List[Datasource]:
        data = self.api.get('/datasources').json()
        ds_list = []
        for item in data:
            # TODO skip not sql skills
            if item.get('engine') is None:
                continue
            ds_list.append(Datasource(**item))
        return ds_list

    def get(self, name: str) -> Datasource:
        data = self.api.get(f'/datasources/{name}').json()
        # TODO skip not sql skills
        if data.get('engine') is None:
            raise RuntimeError(f'Datasource {name} is not found')
        return Datasource(**data)

    def drop(self, name: str):
        self.api.delete(f'/datasources/{name}')



import requests


def _raise_for_status(response):
    # show response text in error
    if 400 <= response.status_code < 600:
        raise requests.HTTPError(f'{response.reason}: {response.text}', response=response)


class RestAPI:
    def __init__(self, api_key, base_url=None):
        if base_url is None:
            base_url = 'https://mdb.ai/'

        base_url.rstrip('/')
        if not base_url.endswith('/api'):
            base_url = base_url + '/api'
        self.api_key = api_key
        self.base_url = base_url

    def _headers(self):
        return {'Authorization': 'Bearer ' + self.api_key}

    def get(self, url):
        resp = requests.get(self.base_url + url, headers=self._headers())

        _raise_for_status(resp)
        return resp

    def delete(self, url):
        resp = requests.delete(self.base_url + url, headers=self._headers())

        _raise_for_status(resp)
        return resp

    def post(self, url, data):
        resp = requests.post(
            self.base_url + url,
            headers=self._headers(),
            json=data,
        )

        _raise_for_status(resp)
        return resp

    def patch(self, url, data):
        resp = requests.patch(
            self.base_url + url,
            headers=self._headers(),
            json=data,
        )

        _raise_for_status(resp)
        return resp




from minds.rest_api import RestAPI

from minds.datasources import Datasources
from minds.minds import Minds


class Client:

    def __init__(self, api_key, base_url=None):

        self.api = RestAPI(api_key, base_url)

        self.datasources = Datasources(self)

        self.minds = Minds(self)
